from plone.app.testing import PloneSandboxLayer
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting

class UserAndGroupSelectionWidgetFixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import Products.UserAndGroupSelectionWidget
        self.loadZCML(package=Products.UserAndGroupSelectionWidget)

    def setUpPloneSite(self, portal):
        self.applyProfile(portal, 'Products.UserAndGroupSelectionWidget:default')


USERANDGROUPSELECTIONWIDGET_FIXTURE = UserAndGroupSelectionWidgetFixture()
USERANDGROUPSELECTIONWIDGET_INTEGRATION_TESTING = IntegrationTesting(
    bases=(USERANDGROUPSELECTIONWIDGET_FIXTURE,),
    name="UserAndGroupSelectionWidgetFixture:Integration")

USERANDGROUPSELECTIONWIDGET_FUNCTIONAL_TESTING = \
        FunctionalTesting(
                bases=(USERANDGROUPSELECTIONWIDGET_FIXTURE,), 
                name="UserAndGroupSelectionWidgetFixture:Functional"
                )




from zope.interface import Interface
from plone.app.z3cform.interfaces import IPloneFormLayer

class ITestingLayer(Interface):
    """ Marker interface for requests indicating the package has been installed.
        
        This layer is only for testing purposes, (i.e to register
        schema-extenders) and has no production purpose.
    """

class ICustomFormLayer(IPloneFormLayer):
    """ Request layer installed via browserlayer.xml
    """

class IGenericGroupTranslation(Interface):
    """Utils to translate a generic name to a group name.
    """
    
    def translateToRealGroupId(genericgroup):
        """Takes generic group and translate it to a real plone group id.
        
        In case of an unmatching genericgroup this function MUST return the
        original genericgroup.
        """

class IGenericFilterTranslation(Interface):
    """Utils to translate a generic name to a group name.
    """
    
    def translateToFilterDefinition(genericfilter):
        """Takes generic filter and translate it to a real filter definition.
        
        In case of an unmatching genericfilter this function MUST return the
        original genericfilter.
        """


class IUserAndGroupSelectView(Interface):
    """View class interface for the selection popup.
    """
    
    def getUserOrGroupTitle(id):
        """Return either the fullname of the user or the grou title.
        """


class IUserAndGroupSelectPopupView(Interface):
    """View class interface for the selection popup.
    """
    
    def initialize():
        """Initialize the view class.
        """
    
    def getObjectUrl():
        """Return the url of the current object.
        """
    
    def getQueryUrl(**kwargs):
        """Return the current query url.
        """
    
    def isSelected(param, value):
        """Return True if the given value of param was sent by the request.
        """
    
    def getGroupsForPulldown():
        """Return the plone groups.
        """
    
    def getBatch():
        """Return a AlphaBatch object.
        """
    
    def usersOnly():
        """Return wether users only flag is set on widget or not.
        """
    
    def groupsOnly():
        """Return wether groups only flag is set on widget or not.
        """
    
    def multiValued():
        """Return wether field is multivalued or not.
        """





import logging
from Acquisition import aq_parent
from Products.CMFCore.utils import getToolByName

log = logging.getLogger(__name__)

def installFormLayer(context):
    siteroot = aq_parent(context)
    portal_setup = getToolByName(siteroot, 'portal_setup')
    portal_setup.runImportStepFromProfile(
                'profile-Products.UserAndGroupSelectionWidget:default', 
                'browserlayer', 
                run_dependencies=False, 
                purge_old=None)
    log.info('Succesfully imported browserlayer.xml')



import zope.schema
import zope.interface
import z3c.form

from Products.UserAndGroupSelectionWidget.z3cform import widget


class ITestForm(zope.interface.Interface):

    field1 = zope.schema.TextLine(
        title=u"Field1",
        required=True,
        )

    field2 = zope.schema.List(
        title=u"Field2",
        required=True,
        )


class TestForm(z3c.form.form.Form):

    fields = z3c.form.field.Fields(ITestForm)
    fields['field1'].widgetFactory = widget.UserAndGroupSelectionFieldWidget
    fields['field2'].widgetFactory = widget.UserAndGroupSelectionFieldWidget

    ignoreContext = True

    @z3c.form.button.buttonAndHandler(u'Cancel')
    def handleCancel(self, action):
        print 'Canceling'

    @z3c.form.button.buttonAndHandler(u'Save')
    def handleSave(self, action):
        data, errors = self.extractData()
        if errors:
            return False

        print 'Field1: ' + data['field1']
        print 'Field2: ' + data['field2']



import types
import operator

from alphabatch import AlphaBatch

from Acquisition import aq_inner
from ZTUtils import make_query

from zope.interface import implements
from zope.component import queryUtility 
from zope.schema.interfaces import ICollection

from z3c.form.widget import FieldWidget
from z3cform.widget import UserAndGroupSelectionWidget

from plone.dexterity.interfaces import IDexterityFTI
from plone.dexterity import utils

from Products.Archetypes.utils import shasattr
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from Products.PlonePAS.interfaces.group import IGroupIntrospection

from interfaces import IUserAndGroupSelectView
from memberlookup import MemberLookup


class UserAndGroupSelectView(BrowserView):
    """See interfaces.IUserAndGroupSelectView for documentation details.
    """

    def getUserOrGroupTitle(self, id):
        pas = getToolByName(self.context, 'acl_users')
        user = pas.getUserById(id)

        if user is not None:
            fullname = self._getPropertyForMember(user, 'fullname')
            return fullname or id

        for pluginid, plugin in pas.plugins.listPlugins(IGroupIntrospection):
            group = plugin.getGroupById(id)
            if group is not None:
                title = self._getPropertyForMember(group, 'title')
                return title or id

        return id

    def _getPropertyForMember(self, member, propertyname):
        propsheets = member.listPropertysheets()
        for propsheettitle in propsheets:
            propsheet = member.getPropertysheet(propsheettitle)
            property = propsheet.getProperty(propertyname, None)
            if property:
                return property

        return None


class UserAndGroupSelectPopupView(BrowserView):
    """See interfaces.IUserAndGroupSelectPopupView for documentation details.
    """
    implements(IUserAndGroupSelectView)

    def initialize(self):
        """Initialize the view class."""
        fieldId = self.request.get('fieldId','').split('-')[-1]
        typeOrDottedname = self.request.get('typeOrDottedname')
        context = aq_inner(self.context)
        if typeOrDottedname == context.portal_type and shasattr(context, 'Schema'):
            # Archetype
            field = context.Schema().getField(fieldId)
            self.multivalued = field.multiValued
            self.widget = field.widget
        else: 
            fti = queryUtility(IDexterityFTI, name=typeOrDottedname)
            if fti is None:
                # Must be a standalone z3c.form forms then.
                klass = utils.resolveDottedName(typeOrDottedname)
                field = klass(self.context, self.request).fields.get(fieldId).field
                self.widget = FieldWidget(field, UserAndGroupSelectionWidget(field, self.request))
                self.multivalued = ICollection.providedBy(field)
            else:
                # Dexterity
                schema = fti.lookupSchema()
                field = schema.get(fieldId)
                if field is None:
                    # The field might be defined in a behavior schema.
                    # Get the behaviors from either the context or the
                    # portal_type (but not both at the same time).
                    if self.request.get('ignoreContext'):
                        context = None
                        portal_type = typeOrDottedname
                    else:
                        portal_type = None
                    for behavior_schema in \
                            utils.getAdditionalSchemata(context, portal_type):
                        if behavior_schema is not None:
                            field = behavior_schema.get(fieldId)
                            if field is not None:
                                    break
                self.widget = FieldWidget(field, UserAndGroupSelectionWidget(field, self.request))
                self.multivalued = ICollection.providedBy(field)

        self.memberlookup = MemberLookup(self.context,
                                         self.request,
                                         self.widget)


    def getObjectUrl(self):
        r = '%s/%s' % (self.context.absolute_url(), 'userandgroupselect_popup')
        return r

    def getQueryUrl(self, **kwargs):
        baseUrl = self.context.absolute_url()
        if self.request.get('fieldId', '') != '':
            baseUrl += '/userandgroupselect_popup'
        query = self._getQueryString(**kwargs)
        url = '%s?%s' % (baseUrl, query)
        return url

    def isSelected(self, param, value):
        param = self.request.get(param)
        if param:
            if param is types.StringType:
                param = [param]
            if value in param:
                return True
        return False

    def getGroupsForPulldown(self):
        ret = [('ignore', '-')]
        groups = self.memberlookup.getGroups()
        return ret + sorted(groups, key=operator.itemgetter(1))

    def getBatch(self):
        members = self.memberlookup.getMembers()
        return AlphaBatch(members, self.context, self.request)

    def usersOnly(self):
        return self.widget.usersOnly

    def groupsOnly(self):
        return self.widget.groupsOnly

    def multiValued(self):
        if self.multivalued:
            return 1
        return 0

    def _getQueryString(self, **kwargs):
        params = dict()
        for key in self.request.form.keys():
            params[key] = self.request.form[key]
        params.update(kwargs)
        query = make_query(params)
        return query



# -*- coding: utf-8 -*

import Globals
try:
    from collections import OrderedDict
except ImportError:  # pragma: no coverage
    from ordereddict import OrderedDict
from AccessControl import ClassSecurityInfo
from Products.CMFPlone.utils import safe_unicode


class AlphaBatch(object):
    """Object used to batch results alphabetically.
    """
    
    security = ClassSecurityInfo()
    
    JOKER = '*'

    vocab = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
             'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
             JOKER]
             
    threshold = 30
    
    
    def __init__(self, results, context, request):
        """Take the results and the request.
        """
        self.context = context
        self.results = results
        self.currentresults = []
        self.pagemap = OrderedDict()
        self.request = request
        self.showBatch = True
                
        if len(results) < self.threshold:
            self.showBatch = False
            self.currentresults = results
        else:
            self.initialize()
    
    def initialize(self):
        """Initialize this batch object.
        """
        current = self.request.get('currentPage', None)
        pointer = 0
        hasResults = len(self.results)
        nonresults = []

        for term in self.vocab:
            
            # full init for term
            self.pagemap[term] = dict()
            self.pagemap[term]['value'] = term
            self.pagemap[term]['visible'] = False
            self.pagemap[term]['current'] = False
            
            # special handling for joker
            if term == self.JOKER:
                continue

            # assume alpha sorted results here
            for result in self.results[pointer:]:                
                title = safe_unicode(result['fullname']).upper()
                # title.replace(u'ü',u'Ü')
                # title.replace(u'ö',u'Ö')
                # title.replace(u'ä',u'Ä')
                
                currentTerm = title and title[0] or None
                if currentTerm is None or currentTerm not in self.vocab:
                    nonresults.append(result)
                    pointer += 1
                    continue
                
                if title.startswith(term):
                    self.pagemap[term]['visible'] = True
                    if current is None:
                        current = term
                    if term == current:
                        self.currentresults.append(result)
                    pointer += 1
                else:
                    break
            
            # check for current after processing
            if term == current:
                self.pagemap[term]['current'] = True

        if nonresults:
            self.pagemap[self.JOKER]['visible'] = True
        if current == '*':
            self.currentresults = nonresults
            self.pagemap[self.JOKER]['current'] = True
            
    
    security.declarePublic('showBatch') 
    def showBatch(self):
        """Return True if results reaches threshold.
        """
        return self.showBatch
    
    security.declarePublic('getPages')    
    def getPages(self):
        """Return a list of dicts containing page definitions.
        """
        return self.pagemap.values()
    
    security.declarePublic('getResults')
    def getResults(self):
        """Return the current result.
        """
        return self.currentresults

Globals.InitializeClass(AlphaBatch) 



import types
from zope.component import ComponentLookupError
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Widget import TypesWidget
from Products.Archetypes.Registry import registerWidget
from Products.UserAndGroupSelectionWidget.interfaces import IGenericGroupTranslation

class UserAndGroupSelectionWidget(TypesWidget):
    _properties = TypesWidget._properties.copy()
    _properties.update({
        'macro'                 : "userandgroupselect",
        'helper_js'             : ('userandgroupselect.js',),
        'size'                  : 7,    # size of form-element taking the users
        'groupName'             : '',    # takes the given group as default, 
                                         # a group id
        'usersOnly'             : False, # only allow user selection
        'groupsOnly'            : False, # allow only group selection
        'groupIdFilter'         : '*',   # allow all groups
        'searchableProperties'  : (),    # which properties you want to search as well
                                         # eg. ('email', 'fullname', 'location')
        })

    security = ClassSecurityInfo()

    security.declarePublic('process_form')
    def process_form(self, instance, field, form, empty_marker=None, 
                     emptyReturnsMarker=None,):
        """process the form data and return it."""
        result = TypesWidget.process_form (self, instance, field, form, 
                                           empty_marker, emptyReturnsMarker, )
        if result is empty_marker:
            return result
        value, kwargs = result

        # The widget always returns a empty item (strange) when we use the 
        # multival option.
        # Remove the empty items manually
        if type(value) is types.ListType:
            value = [item for item in value if item]
        return value, kwargs

    security.declarePublic('getGroupId')
    def getGroupId(self, instance):
        groupid = self.groupName
        try:
            translator = IGenericGroupTranslation(instance)
        except ComponentLookupError:
            pass
        except TypeError, e:
            if e[0] == 'Could not adapt':
                pass
            else:
                raise
        else:
            groupid = translator.translateToRealGroupId(self.groupName)
        return groupid


registerWidget(
    UserAndGroupSelectionWidget,
    title='User and Group Selection Widget',
    description=('You can select users searched from a popup window.'),
    used_for=('Products.Archetypes.Field.LinesField',
              'Products.Archetypes.Field.StringField', ))




import time
import types
from zope.component import ComponentLookupError

from Products.CMFCore.utils import getToolByName
from Products.PlonePAS.interfaces.group import IGroupIntrospection

from bda.cache import ICacheManager
from bda.cache import Memcached
 
from interfaces import IGenericGroupTranslation
from interfaces import IGenericFilterTranslation

# make this dynamic, XXX
CACHEPROVIDER = Memcached(['127.0.0.1:11211'])

class MemberLookup(object):
    """This object contains the logic to list and search for users and groups.
    """
    
    def __init__(self, context, request, widget):
        """Construct this object and do base initialization.
        """
        self.context = context
        self.widget = widget
        self.searchabletext = request.get('searchabletext', '')
        group = request.get('selectgroup', '')
        try:
            grouptranslation = IGenericGroupTranslation(self.context)
            self.currentgroupid = grouptranslation.translateToRealGroupId(group)
        except ComponentLookupError:
            self.currentgroupid = group
        except TypeError, e:
            if e[0] == 'Could not adapt':
                self.currentgroupid = group
            else:
                raise
        return            
        
    def getGroups(self):
        """Return the groups.
        """
        #start = time.time()
        filter = self._allocateFilter()
        aclu = getToolByName(self.context, 'acl_users')
        groups = aclu.getGroups()
        ret = []
        for group in groups:
            gid = group.getId()
            if not self._groupIdFilterMatch(gid, filter):
                continue
            ret.append((gid, group.getGroupTitleOrName()))
        #print 'getGroups took %s' % str(time.time() - start)
        return ret
        
    def getMembers(self):
        """Return the Users in the following form.
        
        {
            'id': 'mmustermann',
            'fullname': 'Max Mustermann',
        }
        """
        #start = time.time()
        filter = self._allocateFilter()
        group = self.currentgroupid
        if group != 'ignore' and group != '':
            key = 'userandgroupselectionwidget:%s' % group
            manager = ICacheManager(CACHEPROVIDER)
            if isinstance(key, unicode):
                # The CacheManager can't handle unicode
                key = key.encode('utf-8')
            users = manager.getData(self._readGroupMembers, key, args=[group])
        else:
            users = self._searchUsers()
        reduce = True
        for fil in filter:
            if fil == '*':
                reduce == False
        if reduce:
            users = self._reduceMembers(users, filter)        
        return users
    
    def _getUserIdsOfGroup(self, groupid):
        aclu = getToolByName(self.context, 'acl_users')
        for id, giplugin in aclu.plugins.listPlugins(IGroupIntrospection):
            userids = giplugin.getGroupMembers(groupid)
            if userids: 
                return userids
        return []
    
    def _readGroupMembers(self, gid):
        aclu = getToolByName(self.context, 'acl_users')
        user_ids = self._getUserIdsOfGroup(gid)
        return self._getUserDefs(user_ids)
    
    def _searchUsers(self):
        # TODO: Search is done over all available groups, not only over groups
        # which should be applied. also see getGroups.

        # search terms of less then 3 chars return empty list
        if len(self.searchabletext) < 3:
            return []

        # search for usernames
        acl_users = getToolByName(self.context, 'acl_users')
        user_ids = [user['id'] for user in acl_users.searchUsers(
            name=self.searchabletext)]

        # search for properties
        if self.widget.searchableProperties:
            membership = getToolByName(self.context, 'portal_membership')
            memberdata = getToolByName(self.context, 'portal_memberdata')
            for user_id in memberdata._members.keys():
                user = membership.getMemberById(user_id)
                if user is not None:
                    for member_property in self.widget.searchableProperties:
                        searched = user.getProperty(member_property, None)
                        if searched is not None and \
                           searched.lower().find(self.searchabletext) != -1:
                            user_ids.append(user.getId())

        return self._getUserDefs(user_ids)
    
    def _getUserDefs(self, uids):
        aclu = getToolByName(self.context, 'acl_users')
        users = [aclu.getUserById(user_id) for user_id in uids]
        ret = []
        for user in users:
            if user is None:
                continue
            user_id = user.getId()
            user_fn = None
            for psheet in user.getOrderedPropertySheets():
                if psheet.hasProperty('fullname'):
                    user_fn = psheet.getProperty('fullname')
                    if user_fn:
                        # do not search other sheets
                        break
            user_fn = user_fn or user_id
            entry = {
                'id': user_id,
                'fullname': user_fn,
            }
            ret.append(entry)
        ret.sort(cmp=lambda x, y: \
            x['fullname'].lower() > y['fullname'].lower() and 1 or -1)
        return ret
    
    def _allocateFilter(self):
        filter = self.widget.groupIdFilter
        try:
            filtertranslation = IGenericFilterTranslation(self.context)
            filter = filtertranslation.translateToFilterDefinition(filter)
        except ComponentLookupError:
            pass
        except TypeError, e:
            if e[0] == 'Could not adapt':
                pass
            else:
                raise        
        if type(filter) in types.StringTypes:
            filter = [filter,]
        return filter

    def _groupIdFilterMatch(self, gid, filter):
        """Check if gid matches filter.
        """
        for fil in filter:
            # wildcard match
            if fil.find('*') != -1:
                # all groups are affected
                if fil == '*':
                    return True
                # wildcard matches like '*foo'
                elif fil.startswith('*'):
                    if gid.endswith(fil[1:]):
                        return True
                # wildcard matches like 'foo*'
                elif fil.endswith('*'):
                    if gid.startswith(fil[:-1]):
                        return True
                # wildacard matches like '*foo*'
                else:
                    if gid.find(fil[1:-1]) != -1:
                        return True
            # exact match
            else:
                if fil == gid:
                    return True
        return False
    
    def _reduceMembers(self, members, filter):
        """Reduce members to match filter.
        """
        # TODO
        return members



import z3c.form

class IUserAndGroupSelectionWidget(z3c.form.interfaces.IWidget):
        """ UserAndGroupSelection widget marker for z3c.form
        """

class IUsersAndGroupsSelectionWidget(z3c.form.interfaces.IMultiWidget):
        """ UsersAndGroupsSelection widget marker for z3c.form
        """



import zope.schema
import zope.interface
import zope.component

import z3c.form.browser.multi
import z3c.form.browser.text
import z3c.form.interfaces
import z3c.form.widget
from z3c.form import interfaces

from Acquisition import aq_base

from Products.UserAndGroupSelectionWidget.interfaces import IGenericGroupTranslation
from interfaces import IUserAndGroupSelectionWidget
from interfaces import IUsersAndGroupsSelectionWidget

class Mixin(object):
    """ """

    def ignoreContext(self):
        """ We need to differentiate between z3c.form forms that are bound to
            content types (i.e dexterity) and those that aren't.
        """
        if hasattr(aq_base(self.form), 'ignoreContext'):
            return self.form.ignoreContext
        return False

    @property
    def typeOrDottedname(self):
        try:
            return self.form.portal_type
        except AttributeError:
            return '%s.%s' % (
                        self.form.__module__, 
                        self.form.__class__.__name__)

    def getGroupId(self, instance):
        groupid = self.groupName
        try:
            translator = IGenericGroupTranslation(instance)
        except zope.component.ComponentLookupError:
            pass
        except TypeError, e:
            if e[0] == 'Could not adapt':
                pass
            else:
                raise TypeError(e)
        else:
            groupid = translator.translateToRealGroupId(self.groupName)
        return groupid


class UserAndGroupSelectionWidget(Mixin, z3c.form.browser.text.TextWidget):
    """ A single-valued user or group selection widget for z3c.form
    """
    zope.interface.implementsOnly(IUserAndGroupSelectionWidget)

    macro = "userandgroupselect"
    helper_js = ('userandgroupselect.js',)
    size = 8        # size of form-element taking the users
    groupName = ''  # takes the given group as default, a group id
    usersOnly = False     # only allow user selection
    groupsOnly = False    # allow only group selection
    groupIdFilter = '*'   # allow all groups
    searchableProperties = ()    # which properties you want to search as well
                                 # eg. ('email', 'fullname', 'location')

    def __init__(self, field, request):
        super(UserAndGroupSelectionWidget, self).__init__(request)


class UsersAndGroupsSelectionWidget(Mixin, z3c.form.browser.multi.MultiWidget):
    """ A multi-valued users and/or groups selection widget for z3c.form
    """
    zope.interface.implementsOnly(
                    IUsersAndGroupsSelectionWidget,
                    z3c.form.interfaces.IButtonForm, 
                    z3c.form.interfaces.IHandlerForm)

    klass = u'users-and-groups-selection-widget'
    helper_js = ('userandgroupselect.js',)
    size = 8        # size of form-element taking the users
    groupName = ''  # takes the given group as default, a group id
    usersOnly = False     # only allow user selection
    groupsOnly = False    # allow only group selection
    searchableProperties = ()   # which properties you want to search as well
                                # eg. ('email', 'fullname', 'location')

    def extract(self, default=z3c.form.interfaces.NO_VALUE):
        return self.request.get(self.name, default)

    def update(self):
        return super(z3c.form.browser.multi.MultiWidget, self).update()


@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
@zope.component.adapter(zope.schema.interfaces.IField,
                        z3c.form.interfaces.IFormLayer)
def UserAndGroupSelectionFieldWidget(field, request):
    """IFieldWidget factory for UserAndGroupSelectionWidget
    """
    return z3c.form.widget.FieldWidget(field,
        UserAndGroupSelectionWidget(field, request))

@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
def UsersAndGroupsSelectionWidgetFactory(field, request):
    """IFieldWidget factory for TextLinesWidget."""
    return z3c.form.widget.FieldWidget(
                field, UsersAndGroupsSelectionWidget(request))

@zope.interface.implementer(z3c.form.interfaces.IFieldWidget)
@zope.component.adapter(zope.schema.interfaces.IField,
                        z3c.form.interfaces.IFormLayer)
def UsersAndGroupsSelectionFieldWidget(field, request):
    """IFieldWidget factory for UserAndGroupSelectionWidget
    """
    return UsersAndGroupsSelectionWidgetFactory(field, request)




import unittest
import zope.app.testing.placelesssetup
import zope.testing.doctest

def test_suite():
    return unittest.TestSuite((
            zope.testing.doctest.DocFileSuite(
                'z3cform/DOCTESTS.txt',
                package='Products.UserAndGroupSelectionWidget',
                setUp=zope.app.testing.placelesssetup.setUp,
                tearDown=zope.app.testing.placelesssetup.tearDown),
            ))



from zope.component import adapts
from zope.interface import implements
from archetypes.schemaextender.field import ExtensionField
from archetypes.schemaextender.interfaces import IBrowserLayerAwareExtender
from archetypes.schemaextender.interfaces import IOrderableSchemaExtender
from Products.ATContentTypes.interface import IATDocument
from Products.Archetypes import atapi
from Products.UserAndGroupSelectionWidget.interfaces import ITestingLayer
from Products.UserAndGroupSelectionWidget.at import widget

class StringField(ExtensionField, atapi.StringField):
    """A string field."""

class LinesField(ExtensionField, atapi.LinesField):
    """A lines field."""

class PageExtender(object):
    adapts(IATDocument)
    layer = ITestingLayer
    implements(IOrderableSchemaExtender, IBrowserLayerAwareExtender)

    fields = [
        StringField("field1",
            widget = widget.UserAndGroupSelectionWidget(
                label="Field1",
                ),
            ),
        LinesField("field2",
            widget = widget.UserAndGroupSelectionWidget(
                label="Field2",
                ),
            ),
        ]

    def __init__(self, context):
        self.context = context

    def getFields(self):
        return self.fields




# Copyright 2008-2009, BlueDynamics Alliance, Austria - http://bluedynamics.com
# BSD derivative License 

import sys
from setuptools import setup
from setuptools import find_packages


version = '3.0b5'

if sys.version_info < (2, 7):
    extra_install_requires = ['ordereddict']
else:
    extra_install_requires = []

setup(
    name='Products.UserAndGroupSelectionWidget',
    version=version,
    description='Archetypes Widget for User and Group Selection, works '
                'with many users.',
    long_description=open('README.rst').read() + open('HISTORY.txt').read() + open('LICENSE.txt').read(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Zope2',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        ], 
    keywords='zope zope2 plone archetypes z3cform widget user',
    author='BlueDynamics Alliance',
    author_email='dev@bluedynamics.com',
    url='https://github.com/collective/Products.UserAndGroupSelectionWidget',
    license='BSD',
    packages=find_packages('src'),
    package_dir = {'': 'src'},
    namespace_packages=['Products', ],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'zope.interface',
        'zope.component',
        'zope.schema>=4.1.1',
        'Zope2',
        'AccessControl',
        'Products.CMFCore',
        'Products.CMFPlone',
        'Products.PlonePAS',
        'bda.cache',
        'lxml',
        'plone.dexterity',
        ] + extra_install_requires,
    extras_require = {
        'test': [
            'unittest2',
            'zope.app.testing',
            'plone.app.testing',
            ],
        },
    )



import unittest2 as unittest

from Products.CMFCore.utils import getToolByName
from Products.UserAndGroupSelectionWidget import testing
from Products.UserAndGroupSelectionWidget.memberlookup import MemberLookup


class MemberLookupTest(unittest.TestCase):

    layer = testing.USERANDGROUPSELECTIONWIDGET_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        class FakeWidget(object):
            groupIdFilter = []
            searchableProperties = []

        self.widget = FakeWidget()
        self.widget.searchableProperties = ('email', 'fullname',
                'home_page', 'location', 'description')

        username = 'example-user'
        acl_users = getToolByName(self.portal, 'acl_users')
        acl_users.userFolderAddUser(username, 'secret', ['Member'], [])

        membership = getToolByName(self.portal, 'portal_membership')
        self.member = membership.getMemberById(username)
        self.member.setMemberProperties(mapping={
            'email': 'test@exampLe1.com',
            'fullname': 'Example2 User',
            'home_page': 'http://exaMple3.com',
            'location': 'Example4',
            'description': 'Example5 description',
            })
        self.memberlookup = MemberLookup(self.portal, self.request, self.widget)

    def test_search_user_via_username(self):
        self.memberlookup.searchabletext = 'example-'
        members = self.memberlookup.getMembers()

        self.assertEqual(1, len(members))
        self.assertEqual(self.member.getId(), members[0]['id'])

    def test_search_user_via_email(self):
        self.memberlookup.searchabletext = 'example1'
        members = self.memberlookup.getMembers()

        self.assertEqual(1, len(members))
        self.assertEqual(self.member.getId(), members[0]['id'])

    def test_search_user_via_fullname(self):
        self.memberlookup.searchabletext = 'example2'
        members = self.memberlookup.getMembers()

        self.assertEqual(1, len(members))
        self.assertEqual(self.member.getId(), members[0]['id'])

    def test_search_user_via_homepage(self):
        self.memberlookup.searchabletext = 'example3'
        members = self.memberlookup.getMembers()

        self.assertEqual(1, len(members))
        self.assertEqual(self.member.getId(), members[0]['id'])

    def test_search_user_via_location(self):
        self.memberlookup.searchabletext = 'example4'
        members = self.memberlookup.getMembers()

        self.assertEqual(1, len(members))
        self.assertEqual(self.member.getId(), members[0]['id'])

    def test_search_user_via_description(self):
        self.memberlookup.searchabletext = 'example5'
        members = self.memberlookup.getMembers()

        self.assertEqual(1, len(members))
        self.assertEqual(self.member.getId(), members[0]['id'])


    def test_disable_search_over_user_properties(self):
        self.widget.searchableProperties = ()
        memberlookup = MemberLookup(self.portal, self.request, self.widget)
        memberlookup.searchabletext = 'example1'

        self.assertEqual(0, len(memberlookup.getMembers()))


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)
